import os
from zc.async.testing import wait_for_result
from zope.interface.verify import verifyClass, verifyObject
from zope.component import getUtility
from waeup.kofa.interfaces import IJobManager
from waeup.kofa.students.reports.level_report import ILevelReport
from waeup.kofa.students.tests.test_catalog import CatalogTestSetup
from waeup.kofa.students.tests.test_browser import StudentsFullSetup
from waeup.kofa.browser.tests.test_pdf import samples_dir

from waeup.aaue.students.reports.level_report import (
    CustomLevelReport, GradLevelReport)

from waeup.aaue.testing import FunctionalLayer

class LevelReportTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_iface(self):
        # ensure we fullfill interface contracts
        obj = CustomLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        verifyClass(ILevelReport, CustomLevelReport)
        verifyObject(ILevelReport, obj)
        return

    def test_get_students(self):
        # we can get a table with one student
        self.student['studycourse']['100']['Course1'].ca = 0
        lr =  CustomLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        result = lr._get_students('fac1', 'dep1', 'CERT1', 2010, 100)
        self.assertEqual(result,
            [(u'1234', u'TESTER, Bob', 30, 30, '5.000',
               'Nil', 'Nil', 30, 30, '5.000', 'Proceed')])
        return

    def test_create_pdf(self):
        self.student['studycourse']['100']['Course1'].ca = 0
        self.app['faculties']['fac1'].officer_1 = 'Rosa Moonshine - Dean of Faculty'
        self.app['faculties']['fac1']['dep1'].officer_1 = 'Amalia Sunshine - Head of Department'
        report = CustomLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        result = report.create_pdf('JOB_ID')
        self.assertTrue(result.startswith('%PDF-'))
        path = os.path.join(samples_dir(), 'level_report.pdf')
        open(path, 'wb').write(result)
        print "Sample level_report.pdf written to %s" % path
        return

class GradLevelReportTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_iface(self):
        # ensure we fullfill interface contracts
        obj = CustomLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        verifyClass(ILevelReport, GradLevelReport)
        verifyObject(ILevelReport, obj)
        return

    def test_get_students(self):
        self.student['studycourse']['100']['Course1'].ca = 0
        lr =  GradLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        # this is not a graduating student
        result = lr._get_students('fac1', 'dep1', 'CERT1', 2010, 100)
        self.assertEqual(result, [])
        # if end level is 100, the student seems graduating and
        # we can get a table with one student
        self.certificate.end_level = 100
        self.student['studycourse']['100']['Course1'].credits = 91
        result = lr._get_students('fac1', 'dep1', 'CERT1', 2010, 100)
        self.assertEqual(result,
            [(u'1234', u'TESTER, Bob', 91, 91, '5.000',
               'Nil', 'Nil', 91, 91, '5.00', '1s_st_s')])
        return

    def test_create_pdf(self):
        self.student['studycourse']['100']['Course1'].ca = 0
        self.student['studycourse']['100']['Course1'].credits = 91
        self.certificate.end_level = 100
        self.app['faculties']['fac1'].officer_1 = 'Rosa Moonshine - Dean of Faculty'
        self.app['faculties']['fac1']['dep1'].officer_1 = 'Amalia Sunshine - Head of Department'
        report = GradLevelReport('fac1', 'dep1', 'CERT1', 2010, 100)
        result = report.create_pdf('JOB_ID')
        self.assertTrue(result.startswith('%PDF-'))
        path = os.path.join(samples_dir(), 'grad_level_report.pdf')
        open(path, 'wb').write(result)
        print "Sample grad_level_report.pdf written to %s" % path
        return
